@@ -13,27 +13,6 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
-package Dpkg::BuildProfiles;
-
-use strict;
-use warnings;
-
-our $VERSION = '1.00';
-our @EXPORT_OK = qw(
-    get_build_profiles
-    set_build_profiles
-    parse_build_profiles
-    evaluate_restriction_formula
-);
-
-use Exporter qw(import);
-
-use Dpkg::Util qw(:list);
-use Dpkg::BuildEnv;
-
-my $cache_profiles;
-my @build_profiles;
-
 =encoding utf8
 
 =head1 NAME
@@ -45,6 +24,29 @@ Dpkg::BuildProfiles - handle build profiles
 The Dpkg::BuildProfiles module provides functions to handle the build
 profiles.
 
+=cut
+
+package Dpkg::BuildProfiles 1.00;
+
+use strict;
+use warnings;
+
+our @EXPORT_OK = qw(
+    get_build_profiles
+    set_build_profiles
+    parse_build_profiles
+    evaluate_restriction_formula
+);
+
+use Exporter qw(import);
+use List::Util qw(any);
+
+use Dpkg::BuildEnv;
+use Dpkg::Vendor qw(run_vendor_hook);
+
+my $cache_profiles;
+my @build_profiles;
+
 =head1 FUNCTIONS
 
 =over 4
@@ -60,9 +62,10 @@ sub get_build_profiles {
     return @build_profiles if $cache_profiles;
 
     if (Dpkg::BuildEnv::has('DEB_BUILD_PROFILES')) {
-        @build_profiles = split /\s+/, Dpkg::BuildEnv::get('DEB_BUILD_PROFILES');
+        @build_profiles = split ' ', Dpkg::BuildEnv::get('DEB_BUILD_PROFILES');
     }
     $cache_profiles = 1;
+    run_vendor_hook('update-buildprofiles', \@build_profiles);
 
     return @build_profiles;
 }
@@ -79,7 +82,8 @@ sub set_build_profiles {
 
     $cache_profiles = 1;
     @build_profiles = @profiles;
-    Dpkg::BuildEnv::set('DEB_BUILD_PROFILES', join ' ', @profiles);
+    run_vendor_hook('update-buildprofiles', \@build_profiles);
+    Dpkg::BuildEnv::set('DEB_BUILD_PROFILES', join ' ', @build_profiles);
 }
 
 =item @profiles = parse_build_profiles($string)
@@ -91,9 +95,9 @@ Parses a build profiles specification, into an array of array references.
 sub parse_build_profiles {
     my $string = shift;
 
-    $string =~ s/^\s*<(.*)>\s*$/$1/;
+    $string =~ s/^\s*<\s*(.*)\s*>\s*$/$1/;
 
-    return map { [ split /\s+/ ] } split />\s+</, $string;
+    return map { [ split ' ' ] } split /\s*>\s+<\s*/, $string;
 }
 
 =item evaluate_restriction_formula(\@formula, \@profiles)
@@ -114,6 +118,7 @@ sub evaluate_restriction_formula {
         foreach my $restriction (@$restrlist) {
             next if $restriction !~ m/^(!)?(.+)/;
 
+            ## no critic (RegularExpressions::ProhibitCaptureWithoutTest)
             my $negated = defined $1 && $1 eq '!';
             my $profile = $2;
             my $found = any { $_ eq $profile } @{$profiles};
