@@ -31,14 +31,13 @@ use Dpkg::Gettext;
 use Dpkg::ErrorHandling;
 use Dpkg::Lock;
 use Dpkg::Arch qw(get_host_arch debarch_eq debarch_is debarch_list_parse);
-use Dpkg::Package;
 use Dpkg::BuildProfiles qw(get_build_profiles);
 use Dpkg::Deps;
 use Dpkg::Control;
 use Dpkg::Control::Info;
 use Dpkg::Control::Fields;
 use Dpkg::Substvars;
-use Dpkg::Vars;
+use Dpkg::Package;
 use Dpkg::Changelog::Parse;
 use Dpkg::Dist::Files;
 
@@ -194,29 +193,30 @@ $substvars->set_msg_prefix(sprintf(g_('package %s: '), $pkg->{Package}));
 
 # Scan source package
 my $src_fields = $control->get_source();
-foreach (keys %{$src_fields}) {
-    if (m/^Source$/) {
-	set_source_package($src_fields->{$_});
-    } elsif (m/^Description$/) {
+foreach my $f (keys %{$src_fields}) {
+    if ($f eq 'Source') {
+        set_source_name($src_fields->{$f});
+    } elsif ($f eq 'Description') {
         # Description in binary packages is not inherited, do not copy this
         # field, only initialize the description substvars.
-        $substvars->set_desc_substvars($src_fields->{$_});
+        $substvars->set_desc_substvars($src_fields->{$f});
     } else {
-        field_transfer_single($src_fields, $fields);
+        field_transfer_single($src_fields, $fields, $f);
     }
 }
 $substvars->set_field_substvars($src_fields, 'S');
 
 # Scan binary package
-foreach (keys %{$pkg}) {
-    my $v = $pkg->{$_};
-    if (field_get_dep_type($_)) {
+foreach my $f (keys %{$pkg}) {
+    my $v = $pkg->{$f};
+
+    if (field_get_dep_type($f)) {
 	# Delay the parsing until later
-    } elsif (m/^Architecture$/) {
+    } elsif ($f eq 'Architecture') {
 	my $host_arch = get_host_arch();
 
 	if (debarch_eq('all', $v)) {
-	    $fields->{$_} = $v;
+            $fields->{$f} = $v;
 	} else {
 	    my @archlist = debarch_list_parse($v, positive => 1);
 
@@ -225,26 +225,26 @@ foreach (keys %{$pkg}) {
 			 "appear in package '%s' architecture list (%s)"),
 		      $host_arch, $oppackage, "@archlist");
 	    }
-	    $fields->{$_} = $host_arch;
+            $fields->{$f} = $host_arch;
 	}
     } else {
-        field_transfer_single($pkg, $fields);
+        field_transfer_single($pkg, $fields, $f);
     }
 }
 
 # Scan fields of dpkg-parsechangelog
-foreach (keys %{$changelog}) {
-    my $v = $changelog->{$_};
+foreach my $f (keys %{$changelog}) {
+    my $v = $changelog->{$f};
 
-    if (m/^Source$/) {
-	set_source_package($v);
-    } elsif (m/^Version$/) {
+    if ($f eq 'Source') {
+        set_source_name($v);
+    } elsif ($f eq 'Version') {
         # Already handled previously.
-    } elsif (m/^Maintainer$/) {
+    } elsif ($f eq 'Maintainer') {
         # That field must not be copied from changelog even if it's
         # allowed in the binary package control information
     } else {
-        field_transfer_single($changelog, $fields);
+        field_transfer_single($changelog, $fields, $f);
     }
 }
 
@@ -328,7 +328,7 @@ if ($pkg_type eq 'udeb') {
     }
 }
 
-my $sourcepackage = get_source_package();
+my $sourcepackage = get_source_name();
 my $binarypackage = $override{'Package'} // $fields->{'Package'};
 my $verdiff = $binaryversion ne $sourceversion;
 if ($binarypackage ne $sourcepackage || $verdiff) {
