@@ -45,7 +45,7 @@ use Dpkg::Control::Tests;
 use Dpkg::Control::Fields;
 use Dpkg::Substvars;
 use Dpkg::Version;
-use Dpkg::Vars;
+use Dpkg::Package;
 use Dpkg::Changelog::Parse;
 use Dpkg::Source::Format;
 use Dpkg::Source::Package qw(get_default_diff_ignore_regex
@@ -160,6 +160,10 @@ while (@options) {
 	usageerr(g_('%s is not a compression level'), $comp_level)
 	    unless compression_is_valid_level($comp_level);
 	compression_set_default_level($comp_level);
+    } elsif (m/^--threads-max=(.*)$/) {
+        my $threads = $1;
+        $options{comp_threads} = $threads;
+        compression_set_threads($threads);
     } elsif (m/^-c(.*)$/) {
         $controlfile = $1;
     } elsif (m/^-l(.*)$/) {
@@ -170,7 +174,9 @@ while (@options) {
         $override{$1} = $2;
     } elsif (m/^-U([^\=:]+)$/) {
         $remove{$1} = 1;
-    } elsif (m/^-(?:i|-diff-ignore(?:$|=))(.*)$/) {
+    } elsif (m/^--diff-ignore$/) {
+        $options{diff_ignore_regex} = $diff_ignore_regex;
+    } elsif (m/^-(?:i|-diff-ignore=)(.*)$/) {
         $options{diff_ignore_regex} = $1 ? $1 : $diff_ignore_regex;
     } elsif (m/^--extend-diff-ignore=(.+)$/) {
 	$diff_ignore_regex .= "|$1";
@@ -224,9 +230,7 @@ unless (defined($options{opmode})) {
 }
 
 if ($options{opmode} =~ /^(build|print-format|(before|after)-build|commit)$/) {
-
     $options{ARGV} = \@ARGV;
-
     $options{changelog_file} ||= "$dir/debian/changelog";
     $controlfile ||= "$dir/debian/control";
 
@@ -268,24 +272,26 @@ if ($options{opmode} =~ /^(build|print-format|(before|after)-build|commit)$/) {
           $controlfile) unless defined $src_fields;
     my $src_sect = $src_fields->{'Section'} || 'unknown';
     my $src_prio = $src_fields->{'Priority'} || 'unknown';
-    foreach (keys %{$src_fields}) {
-	my $v = $src_fields->{$_};
-	if (m/^Source$/i) {
-	    set_source_package($v);
-	    $fields->{$_} = $v;
-	} elsif (m/^Uploaders$/i) {
-	    ($fields->{$_} = $v) =~ s/\s*[\r\n]\s*/ /g; # Merge in a single-line
-	} elsif (m/^Build-(?:Depends|Conflicts)(?:-Arch|-Indep)?$/i) {
+    foreach my $f (keys %{$src_fields}) {
+        my $v = $src_fields->{$f};
+
+        if ($f eq 'Source') {
+            set_source_name($v);
+            $fields->{$f} = $v;
+        } elsif ($f eq 'Uploaders') {
+            # Merge in a single-line.
+            ($fields->{$f} = $v) =~ s/\s*[\r\n]\s*/ /g;
+        } elsif (any { $f eq $_ } field_list_src_dep()) {
 	    my $dep;
-	    my $type = field_get_dep_type($_);
+            my $type = field_get_dep_type($f);
 	    $dep = deps_parse($v, build_dep => 1, union => $type eq 'union');
-	    error(g_('cannot parse %s field'), $_) unless defined $dep;
+            error(g_('cannot parse %s field'), $f) unless defined $dep;
 	    my $facts = Dpkg::Deps::KnownFacts->new();
 	    $dep->simplify_deps($facts);
 	    $dep->sort() if $type eq 'union';
-	    $fields->{$_} = $dep->output();
+            $fields->{$f} = $dep->output();
 	} else {
-            field_transfer_single($src_fields, $fields);
+            field_transfer_single($src_fields, $fields, $f);
 	}
     }
 
@@ -296,7 +302,7 @@ if ($options{opmode} =~ /^(build|print-format|(before|after)-build|commit)$/) {
 	my $sect = $pkg->{'Section'} || $src_sect;
 	my $prio = $pkg->{'Priority'} || $src_prio;
 	my $type = $pkg->{'Package-Type'} ||
-	        $pkg->get_custom_field('Package-Type') || 'deb';
+            $pkg->get_custom_field('Package-Type') || 'deb';
         my $arch = $pkg->{'Architecture'};
         my $profile = $pkg->{'Build-Profiles'};
 
@@ -325,9 +331,10 @@ if ($options{opmode} =~ /^(build|print-format|(before|after)-build|commit)$/) {
 
         push @pkglist, $pkg_summary;
 	push @binarypackages, $p;
-	foreach (keys %{$pkg}) {
-	    my $v = $pkg->{$_};
-            if (m/^Architecture$/) {
+        foreach my $f (keys %{$pkg}) {
+            my $v = $pkg->{$f};
+
+            if ($f eq 'Architecture') {
                 # Gather all binary architectures in one set. 'any' and 'all'
                 # are special-cased as they need to be the only ones in the
                 # current stanza if present.
@@ -345,10 +352,10 @@ if ($options{opmode} =~ /^(build|print-format|(before|after)-build|commit)$/) {
                         push(@sourcearch, $a) unless $archadded{$a}++;
                     }
                 }
-            } elsif (m/^(?:Homepage|Description)$/) {
+            } elsif (any { $f eq $_ } qw(Homepage Description)) {
                 # Do not overwrite the same field from the source entry
             } else {
-                field_transfer_single($pkg, $fields);
+                field_transfer_single($pkg, $fields, $f);
             }
 	}
     }
@@ -381,23 +388,23 @@ if ($options{opmode} =~ /^(build|print-format|(before|after)-build|commit)$/) {
     set_testsuite_fields($fields, @binarypackages);
 
     # Scan fields of dpkg-parsechangelog
-    foreach (keys %{$changelog}) {
-        my $v = $changelog->{$_};
+    foreach my $f (keys %{$changelog}) {
+        my $v = $changelog->{$f};
 
-	if (m/^Source$/) {
-	    set_source_package($v);
-	    $fields->{$_} = $v;
-	} elsif (m/^Version$/) {
+        if ($f eq 'Source') {
+            set_source_name($v);
+            $fields->{$f} = $v;
+        } elsif ($f eq 'Version') {
 	    my ($ok, $error) = version_check($v);
             error($error) unless $ok;
-	    $fields->{$_} = $v;
-	} elsif (m/^Binary-Only$/) {
+            $fields->{$f} = $v;
+        } elsif ($f eq 'Binary-Only') {
 	    error(g_('building source for a binary-only release'))
 	        if $v eq 'yes' and $options{opmode} eq 'build';
-	} elsif (m/^Maintainer$/i) {
+        } elsif ($f eq 'Maintainer') {
             # Do not replace the field coming from the source entry
 	} else {
-            field_transfer_single($changelog, $fields);
+            field_transfer_single($changelog, $fields, $f);
 	}
     }
 
@@ -433,15 +440,13 @@ if ($options{opmode} =~ /^(build|print-format|(before|after)-build|commit)$/) {
 
     # Write the .dsc
     my $dscname = $srcpkg->get_basename(1) . '.dsc';
-    info(g_('building %s in %s'), get_source_package(), $dscname);
+    info(g_('building %s in %s'), get_source_name(), $dscname);
     $srcpkg->write_dsc(filename => $dscname,
 		       remove => \%remove,
 		       override => \%override,
 		       substvars => $substvars);
     exit(0);
-
 } elsif ($options{opmode} eq 'extract') {
-
     # Check command line
     unless (scalar(@ARGV)) {
         usageerr(g_('--%s needs at least one argument, the .dsc'),
@@ -556,12 +561,12 @@ sub setopmode {
 
 sub print_option {
     my $opt = shift;
-    my $help;
 
+    my $help = gettext($opt->{help});
     if (length $opt->{name} > 25) {
-        $help .= sprintf "  %-25s\n%s%s.\n", $opt->{name}, ' ' x 27, $opt->{help};
+        return sprintf "  %-25s\n%s%s.\n", $opt->{name}, ' ' x 27, $help;
     } else {
-        $help .= sprintf "  %-25s%s.\n", $opt->{name}, $opt->{help};
+        return sprintf "  %-25s%s.\n", $opt->{name}, $help;
     }
 }
 
@@ -652,6 +657,8 @@ sub usage {
     get_format_help()
     . "\n" . g_(
 'General options:
+      --threads-max=<threads>
+                           use at most <threads> with compressor.
   -q                       quiet mode.
   -?, --help               show this help message.
       --version            show the version.')
